# swatchjs-batch

An extension for [swatchjs]() to provide a method to invoke multiple API methods
at once.

## Quick start

All you have to do is add a few lines of code to your existing `swatchjs` code.
The sample below shows how to use the `swatchjs-batch` with the simple API from
the `swatchjs`'s README file:

```javascript
const swatch = require('swatchjs');
const batch = require('swatchjs-batch'); // add this line

const api = {
    "numbers.add": (a, b) => Number(a) + Number(b),
    "numbers.sub": (a, b) => Number(a) - Number(b),
};

const model = swatch(api);
const batchModel = swatch({'batch': batch(model)}); // add this line
const extendedModel = model.concat(batchModel); // optionally add this line
```

## API reference

### The `batch` function

```javascript
const batch = require('swatchjs-batch');

const augmentedModel = batch(model, name);
```

Loading this library will result in a function  (`batch` in the example above)
which when called will return a handler that can be used with the `swatch`
function to produce a model for the API method that allows all other methods to
be invoked in a single network call.

The function exposed by the library takes the following parameters:

| Parameter | Required  | Description
|:---       |:---       |:---
| `model`   | Yes       | A model produced by by the `swatch` function (see [swatchjs](https://www.npmjs.com/package/swatchjs#api-reference))
| `name`    | No        | The name of the API method. Defaults to `batch`.

### Return value

The `batch` method returns a handler that takes the following parameters:

| Parameter         | Required  | Description
|:---               |:---       |:---
|`ops`              | Yes       | An array of operations to batch.
|`ops[idx].method`  | Yes       | The method name.
|`ops[idx].args`    | Yes       | An object with the arguments to pass to the method.

The methods are not guaranteed to be executed in any particular order, and they
succeed or fail independently.

The response will be an array of responses, where each response has the
following properties:

| Property  | Present   | Description
|:---       |:---       |:---
|`ok`       | Always    | `true` if the method succeed, `false` otherwise.
|`result`   | Sometimes | Value returned by method handler if it succeeded and returned anything other than `undefined`.
|`error`    | Sometimes | Exception thrown by method handler when it failed.

## Runtime errors

The following errors will be generated during runtime (when invoking the handler):

| Error             | Error scope   | Description
|:---               |:---           |:---
|`invalid_request`  | Handler       | `ops` parameter was not valid.
|`invalid_method`   | Operation     | Specified method was not declared in the API.
