'use strict';

const opsSchema = require('./schemas/ops');
const validator = require('./validator');
const validate = validator(opsSchema);

function batch(model) {
  const methods = {};
  model.forEach(method => void(methods[method.name] = method.handle));
  return handler;

  function handler(ops) {
    try {
      validate(ops);
    } catch (e) {
      throw 'invalid_request';
    }

    const results = ops.map(handleOp);
    return results;
  }

  function handleOp(op) {
    if (op.method in methods) {
      try {
        const result = methods[op.method](op.args);
        return {
          ok: true,
          result,
        };
      } catch(e) {
        return {
          ok: false,
          error: e.message || e,
        };
      }
    } else {
      return {
        ok: false,
        error: 'invalid_method',
      };
    }
  }
}

module.exports = batch;
